@@ -158,8 +158,15 @@ class TestingApp(RequestHandler):
     def redirect(self, request):
         "Perform a redirect to ``target``"
         target = request.params.get('target', '/')
+        status = request.params.get('status', '303 See Other')
+        if len(status) == 3:
+            status = '%s Redirect' % status.decode('latin-1')
+
         headers = [('Location', target)]
-        return Response(status='303 See Other', headers=headers)
+        return Response(status=status, headers=headers)
+
+    def not_found(self, request):
+        return Response('Not found', status='404 Not Found')
 
     def multi_redirect(self, request):
         "Performs a redirect chain based on ``redirect_codes``"
