@@ -1,14 +1,10 @@
-import functools
-import unittest
-from collections import namedtuple
+import socket
+import sys
 
 from urllib3.poolmanager import (
-    _default_key_normalizer,
-    HTTPPoolKey,
-    HTTPSPoolKey,
+    PoolKey,
     key_fn_by_scheme,
     PoolManager,
-    SSL_KEYWORDS,
 )
 from urllib3 import connection_from_url
 from urllib3.exceptions import (
@@ -17,17 +13,25 @@ from urllib3.exceptions import (
 )
 from urllib3.util import retry, timeout
 
+if sys.version_info >= (2, 7):
+    import unittest
+else:
+    import unittest2 as unittest
+
 
 class TestPoolManager(unittest.TestCase):
     def test_same_url(self):
         # Convince ourselves that normally we don't get the same object
         conn1 = connection_from_url('http://localhost:8081/foo')
         conn2 = connection_from_url('http://localhost:8081/bar')
+        self.addCleanup(conn1.close)
+        self.addCleanup(conn2.close)
 
         self.assertNotEqual(conn1, conn2)
 
         # Now try again using the PoolManager
         p = PoolManager(1)
+        self.addCleanup(p.clear)
 
         conn1 = p.connection_from_url('http://localhost:8081/foo')
         conn2 = p.connection_from_url('http://localhost:8081/bar')
@@ -49,6 +53,7 @@ class TestPoolManager(unittest.TestCase):
         connections = set()
 
         p = PoolManager(10)
+        self.addCleanup(p.clear)
 
         for url in urls:
             conn = p.connection_from_url(url)
@@ -58,6 +63,7 @@ class TestPoolManager(unittest.TestCase):
 
     def test_manager_clear(self):
         p = PoolManager(5)
+        self.addCleanup(p.clear)
 
         conn_pool = p.connection_from_url('http://google.com')
         self.assertEqual(len(p.pools), 1)
@@ -75,9 +81,9 @@ class TestPoolManager(unittest.TestCase):
 
         self.assertEqual(len(p.pools), 0)
 
-
     def test_nohost(self):
         p = PoolManager(5)
+        self.addCleanup(p.clear)
         self.assertRaises(LocationValueError, p.connection_from_url, 'http://@')
         self.assertRaises(LocationValueError, p.connection_from_url, None)
 
@@ -107,6 +113,7 @@ class TestPoolManager(unittest.TestCase):
             'source_address': '127.0.0.1',
         }
         p = PoolManager()
+        self.addCleanup(p.clear)
         conn_pools = [
             p.connection_from_url('http://example.com/'),
             p.connection_from_url('http://example.com:8000/'),
@@ -127,29 +134,10 @@ class TestPoolManager(unittest.TestCase):
         )
         self.assertTrue(
             all(
-                isinstance(key, HTTPPoolKey)
+                isinstance(key, PoolKey)
                 for key in p.pools.keys())
         )
 
-    def test_http_pool_key_extra_kwargs(self):
-        """Assert non-HTTPPoolKey fields are ignored when selecting a pool."""
-        p = PoolManager()
-        conn_pool = p.connection_from_url('http://example.com/')
-        p.connection_pool_kw['some_kwarg'] = 'that should be ignored'
-        other_conn_pool = p.connection_from_url('http://example.com/')
-
-        self.assertTrue(conn_pool is other_conn_pool)
-
-    def test_http_pool_key_https_kwargs(self):
-        """Assert HTTPSPoolKey fields are ignored when selecting a HTTP pool."""
-        p = PoolManager()
-        conn_pool = p.connection_from_url('http://example.com/')
-        for key in SSL_KEYWORDS:
-            p.connection_pool_kw[key] = 'this should be ignored'
-        other_conn_pool = p.connection_from_url('http://example.com/')
-
-        self.assertTrue(conn_pool is other_conn_pool)
-
     def test_https_pool_key_fields(self):
         """Assert the HTTPSPoolKey fields are honored when selecting a pool."""
         connection_pool_kw = {
@@ -165,6 +153,7 @@ class TestPoolManager(unittest.TestCase):
             'ssl_version': 'SSLv23_METHOD',
         }
         p = PoolManager()
+        self.addCleanup(p.clear)
         conn_pools = [
             p.connection_from_url('https://example.com/'),
             p.connection_from_url('https://example.com:4333/'),
@@ -190,22 +179,14 @@ class TestPoolManager(unittest.TestCase):
         self.assertTrue(all(pool in conn_pools for pool in dup_pools))
         self.assertTrue(
             all(
-                isinstance(key, HTTPSPoolKey)
+                isinstance(key, PoolKey)
                 for key in p.pools.keys())
         )
 
-    def test_https_pool_key_extra_kwargs(self):
-        """Assert non-HTTPSPoolKey fields are ignored when selecting a pool."""
-        p = PoolManager()
-        conn_pool = p.connection_from_url('https://example.com/')
-        p.connection_pool_kw['some_kwarg'] = 'that should be ignored'
-        other_conn_pool = p.connection_from_url('https://example.com/')
-
-        self.assertTrue(conn_pool is other_conn_pool)
-
     def test_default_pool_key_funcs_copy(self):
         """Assert each PoolManager gets a copy of ``pool_keys_by_scheme``."""
         p = PoolManager()
+        self.addCleanup(p.clear)
         self.assertEqual(p.key_fn_by_scheme, p.key_fn_by_scheme)
         self.assertFalse(p.key_fn_by_scheme is key_fn_by_scheme)
 
@@ -219,6 +200,7 @@ class TestPoolManager(unittest.TestCase):
             'ssl_version': 'SSLv23_METHOD',
         }
         p = PoolManager(5, **ssl_kw)
+        self.addCleanup(p.clear)
         conns = []
         conns.append(
             p.connection_from_host('example.com', 443, scheme='https')
@@ -242,26 +224,29 @@ class TestPoolManager(unittest.TestCase):
     def test_https_connection_from_url_case_insensitive(self):
         """Assert scheme case is ignored when pooling HTTPS connections."""
         p = PoolManager()
+        self.addCleanup(p.clear)
         pool = p.connection_from_url('https://example.com/')
         other_pool = p.connection_from_url('HTTPS://EXAMPLE.COM/')
 
         self.assertEqual(1, len(p.pools))
         self.assertTrue(pool is other_pool)
-        self.assertTrue(all(isinstance(key, HTTPSPoolKey) for key in p.pools.keys()))
+        self.assertTrue(all(isinstance(key, PoolKey) for key in p.pools.keys()))
 
     def test_https_connection_from_host_case_insensitive(self):
         """Assert scheme case is ignored when getting the https key class."""
         p = PoolManager()
+        self.addCleanup(p.clear)
         pool = p.connection_from_host('example.com', scheme='https')
         other_pool = p.connection_from_host('EXAMPLE.COM', scheme='HTTPS')
 
         self.assertEqual(1, len(p.pools))
         self.assertTrue(pool is other_pool)
-        self.assertTrue(all(isinstance(key, HTTPSPoolKey) for key in p.pools.keys()))
+        self.assertTrue(all(isinstance(key, PoolKey) for key in p.pools.keys()))
 
     def test_https_connection_from_context_case_insensitive(self):
         """Assert scheme case is ignored when getting the https key class."""
         p = PoolManager()
+        self.addCleanup(p.clear)
         context = {'scheme': 'https', 'host': 'example.com', 'port': '443'}
         other_context = {'scheme': 'HTTPS', 'host': 'EXAMPLE.COM', 'port': '443'}
         pool = p.connection_from_context(context)
@@ -269,7 +254,7 @@ class TestPoolManager(unittest.TestCase):
 
         self.assertEqual(1, len(p.pools))
         self.assertTrue(pool is other_pool)
-        self.assertTrue(all(isinstance(key, HTTPSPoolKey) for key in p.pools.keys()))
+        self.assertTrue(all(isinstance(key, PoolKey) for key in p.pools.keys()))
 
     def test_http_connection_from_url_case_insensitive(self):
         """Assert scheme case is ignored when pooling HTTP connections."""
@@ -279,21 +264,33 @@ class TestPoolManager(unittest.TestCase):
 
         self.assertEqual(1, len(p.pools))
         self.assertTrue(pool is other_pool)
-        self.assertTrue(all(isinstance(key, HTTPPoolKey) for key in p.pools.keys()))
+        self.assertTrue(all(isinstance(key, PoolKey) for key in p.pools.keys()))
 
     def test_http_connection_from_host_case_insensitive(self):
         """Assert scheme case is ignored when getting the https key class."""
         p = PoolManager()
+        self.addCleanup(p.clear)
         pool = p.connection_from_host('example.com', scheme='http')
         other_pool = p.connection_from_host('EXAMPLE.COM', scheme='HTTP')
 
         self.assertEqual(1, len(p.pools))
         self.assertTrue(pool is other_pool)
-        self.assertTrue(all(isinstance(key, HTTPPoolKey) for key in p.pools.keys()))
+        self.assertTrue(all(isinstance(key, PoolKey) for key in p.pools.keys()))
+
+    def test_assert_hostname_and_fingerprint_flag(self):
+        """Assert that pool manager can accept hostname and fingerprint flags."""
+        fingerprint = '92:81:FE:85:F7:0C:26:60:EC:D6:B3:BF:93:CF:F9:71:CC:07:7D:0A'
+        p = PoolManager(assert_hostname=True, assert_fingerprint=fingerprint)
+        self.addCleanup(p.clear)
+        pool = p.connection_from_url('https://example.com/')
+        self.assertEqual(1, len(p.pools))
+        self.assertTrue(pool.assert_hostname)
+        self.assertEqual(fingerprint, pool.assert_fingerprint)
 
     def test_http_connection_from_context_case_insensitive(self):
         """Assert scheme case is ignored when getting the https key class."""
         p = PoolManager()
+        self.addCleanup(p.clear)
         context = {'scheme': 'http', 'host': 'example.com', 'port': '8080'}
         other_context = {'scheme': 'HTTP', 'host': 'EXAMPLE.COM', 'port': '8080'}
         pool = p.connection_from_context(context)
@@ -301,19 +298,103 @@ class TestPoolManager(unittest.TestCase):
 
         self.assertEqual(1, len(p.pools))
         self.assertTrue(pool is other_pool)
-        self.assertTrue(all(isinstance(key, HTTPPoolKey) for key in p.pools.keys()))
+        self.assertTrue(all(isinstance(key, PoolKey) for key in p.pools.keys()))
 
     def test_custom_pool_key(self):
-        """Assert it is possible to define addition pool key fields."""
-        custom_key = namedtuple('CustomKey', HTTPPoolKey._fields + ('my_field',))
-        p = PoolManager(10, my_field='barley')
+        """Assert it is possible to define a custom key function."""
+        p = PoolManager(10)
+        self.addCleanup(p.clear)
 
-        p.key_fn_by_scheme['http'] = functools.partial(_default_key_normalizer, custom_key)
-        p.connection_from_url('http://example.com')
-        p.connection_pool_kw['my_field'] = 'wheat'
-        p.connection_from_url('http://example.com')
+        p.key_fn_by_scheme['http'] = lambda x: tuple(x['key'])
+        pool1 = p.connection_from_url(
+            'http://example.com', pool_kwargs={'key': 'value'})
+        pool2 = p.connection_from_url(
+            'http://example.com', pool_kwargs={'key': 'other'})
+        pool3 = p.connection_from_url(
+            'http://example.com', pool_kwargs={'key': 'value', 'x': 'y'})
 
         self.assertEqual(2, len(p.pools))
+        self.assertTrue(pool1 is pool3)
+        self.assertFalse(pool1 is pool2)
+
+    def test_override_pool_kwargs_url(self):
+        """Assert overriding pool kwargs works with connection_from_url."""
+        p = PoolManager(strict=True)
+        pool_kwargs = {'strict': False, 'retries': 100, 'block': True}
+
+        default_pool = p.connection_from_url('http://example.com/')
+        override_pool = p.connection_from_url(
+            'http://example.com/', pool_kwargs=pool_kwargs)
+
+        self.assertTrue(default_pool.strict)
+        self.assertEqual(retry.Retry.DEFAULT, default_pool.retries)
+        self.assertFalse(default_pool.block)
+
+        self.assertFalse(override_pool.strict)
+        self.assertEqual(100, override_pool.retries)
+        self.assertTrue(override_pool.block)
+
+    def test_override_pool_kwargs_host(self):
+        """Assert overriding pool kwargs works with connection_from_host"""
+        p = PoolManager(strict=True)
+        pool_kwargs = {'strict': False, 'retries': 100, 'block': True}
+
+        default_pool = p.connection_from_host('example.com', scheme='http')
+        override_pool = p.connection_from_host('example.com', scheme='http',
+                                               pool_kwargs=pool_kwargs)
+
+        self.assertTrue(default_pool.strict)
+        self.assertEqual(retry.Retry.DEFAULT, default_pool.retries)
+        self.assertFalse(default_pool.block)
+
+        self.assertFalse(override_pool.strict)
+        self.assertEqual(100, override_pool.retries)
+        self.assertTrue(override_pool.block)
+
+    def test_pool_kwargs_socket_options(self):
+        """Assert passing socket options works with connection_from_host"""
+        p = PoolManager(socket_options=[])
+        override_opts = [
+            (socket.SOL_SOCKET, socket.SO_REUSEADDR, 1),
+            (socket.IPPROTO_TCP, socket.TCP_NODELAY, 1)
+        ]
+        pool_kwargs = {'socket_options': override_opts}
+
+        default_pool = p.connection_from_host('example.com', scheme='http')
+        override_pool = p.connection_from_host(
+            'example.com', scheme='http', pool_kwargs=pool_kwargs
+        )
+
+        self.assertEqual(default_pool.conn_kw['socket_options'], [])
+        self.assertEqual(
+            override_pool.conn_kw['socket_options'], override_opts
+        )
+
+    def test_merge_pool_kwargs(self):
+        """Assert _merge_pool_kwargs works in the happy case"""
+        p = PoolManager(strict=True)
+        merged = p._merge_pool_kwargs({'new_key': 'value'})
+        self.assertEqual({'strict': True, 'new_key': 'value'}, merged)
+
+    def test_merge_pool_kwargs_none(self):
+        """Assert false-y values to _merge_pool_kwargs result in defaults"""
+        p = PoolManager(strict=True)
+        merged = p._merge_pool_kwargs({})
+        self.assertEqual(p.connection_pool_kw, merged)
+        merged = p._merge_pool_kwargs(None)
+        self.assertEqual(p.connection_pool_kw, merged)
+
+    def test_merge_pool_kwargs_remove_key(self):
+        """Assert keys can be removed with _merge_pool_kwargs"""
+        p = PoolManager(strict=True)
+        merged = p._merge_pool_kwargs({'strict': None})
+        self.assertTrue('strict' not in merged)
+
+    def test_merge_pool_kwargs_invalid_key(self):
+        """Assert removing invalid keys with _merge_pool_kwargs doesn't break"""
+        p = PoolManager(strict=True)
+        merged = p._merge_pool_kwargs({'invalid_key': None})
+        self.assertEqual(p.connection_pool_kw, merged)
 
 
 if __name__ == '__main__':
